#!/bin/bash

#**********************************************
# this script constructs an isotherm based on a free-energy lattice grid
# and a coarse grained potential
#
# note three input files need to be present, the isotherm.pressures file,
# the free energy grid file, and the lattice-discretized solute-coarse grained file
#**********************************************

executable=lattice_MC

if [ ! -f $executable ]
then
echo "";echo " cannot find executable $executable ";echo "";exit
fi

# command line arguments
if (( $# != 3 )) 
then
echo ""; echo "  must have three command line arguments present, the isotherm.pressures file,"
echo "  the free energy grid file, and the lattice-discretized solute-coarse grained potential file ";echo ""
exit
fi

# input files
inputpressures=$1;latticegrid=$2;cggrid=$3;


# get the solute mass and the chemical potentials for the isotherm
chempot=();pressure=();
mass=-1;psection=0
while read line; 
do 
  if [[ $psection == 0 ]]
      then
      flag="$(echo $line | awk '{print $1}')"; 
      if [[ $flag == mass_solute ]]
	  then
	  mass="$(echo $line | awk '{print $2}')";
      elif [[ $flag == pressure* ]]
	  then
	  psection=1
      fi
  else
      press="$(echo $line | awk '{print $1}')";
      pressure+=( $press );
      chem="$(echo $line | awk '{print $2}')";
      chempot+=( $chem );
  fi
done < $inputpressures

# make sure we found mass
  if [[ $mass == -1 ]]
      then
      echo " could not find mass_solute in $inputpressures file" ; exit
  fi


# run the lattice MC simulations

for ((i=0;i<${#chempot[@]};++i));
do 
   ofile="uptake_at_${pressure[i]}bar";
   ./${executable} $latticegrid $cggrid ${chempot[i]} $mass > $ofile
done