module routines
implicit none

contains

  subroutine read_box( ifn )
  use global_variables
  implicit none

    character(100), intent(in) :: ifn
    character(100) :: line
    integer :: n_atom, i_atom, ifile=99, ios
    real*8 :: volume, rtmp(3), tmp1, tmp2, tmp3

    box = 0.0d0
    open( ifile, file=ifn, status='old' )
    read(ifile, *) n_atom
    do i_atom = 1, n_atom
      read(ifile, '(A)') line
    enddo
    read(ifile, *) tmp1, tmp2, tmp3
    read(ifile, *, iostat=ios ) box(2,1), box(2,2), box(2,3)
      if ( ios < 0 ) then ! end of the file
        box = 0.0d0
        box(1,1) = tmp1
        box(2,2) = tmp2
        box(3,3) = tmp3
      else
        box(1,1) = tmp1
        box(1,2) = tmp2
        box(1,3) = tmp3
        read( ifile, * ) box(3,1), box(3,2), box(3,3)
      endif
    close( ifile )

    call cross_product( box(1,:), box(2,:), rtmp(:) )
    volume = dot( box(3,:), rtmp(:) )

    call cross_product(box(1,:), box(2,:), inv_box(:,3))
    call cross_product(box(2,:), box(3,:), inv_box(:,1))
    call cross_product(box(3,:), box(1,:), inv_box(:,2))
    inv_box = inv_box / volume

    return

  end subroutine read_box

  subroutine cross_product( a,b,ans )
    implicit none
    real*8, intent(in) :: a(3),b(3)
    real*8, intent(out) :: ans(3)
    ans(1) =  a(2)*b(3) - a(3)*b(2)
    ans(2) = -a(1)*b(3) + a(3)*b(1)
    ans(3) =  a(1)*b(2) - a(2)*b(1)
  end subroutine cross_product

  function dot( a, b )
    implicit none
    real*8 :: dot
    real*8, dimension(3), intent(in) :: a, b

    dot = a(1)*b(1) + a(2)*b(2) + a(3)*b(3)
    return
  end function dot

  subroutine initialize_potent_table()
  use global_variables
  implicit none

    integer :: i_table
    real*8 :: r, dr

    allocate( potent_table( n_table ) )

    dr = ( re_table - rb_table ) / n_table
    do i_table = 1, n_table
      r = rb_table + ( i_table - 1 ) * dr
      potent_table(i_table) = 4.0d0*epsil*( (sigma/r)**12 - (sigma/r)**6 )
    enddo

  end subroutine initialize_potent_table

  subroutine read_potent_table( ifn )
  use global_variables
  implicit none

    character(100), intent(in) :: ifn
    character(100) :: junk
    integer :: i_table
    real*8 :: r, dr
    integer :: ifile=99
    
    open( ifile, file=ifn, status='old' )
    read(ifile,*) junk, n_table
    allocate( potent_table( n_table ) )
    do i_table = 1, n_table
      read(ifile,*) r, potent_table(i_table)
      if ( i_table == 1 ) then
        rb_table = r
      else if ( i_table == n_table ) then
        re_table = r
      endif
    enddo
    
    close(ifile)
    return
  endsubroutine read_potent_table

  real*8 function potential( r )
  use global_variables
  implicit none
    
    integer :: i_table
    real*8, intent(in) :: r
    real*8 :: dr, s
    
    dr = ( re_table - rb_table ) / n_table
    if ( r < rb_table ) then
      potential = 1000d0
    else if ( r > re_table ) then
      potential = 0.0d0
    else
      s = ( r - rb_table ) / dr
      i_table = floor( s )
      ! linear interpolation
      potential = potent_table(i_table+1)*(i_table+1d0-s) + potent_table(i_table+2)*(s-i_table)
    endif
    return

  end function potential

  ! The input coordinate should be in box direct coordinate
  real*8 function pbc_dr( s1, s2 ) 
  use global_variables
  implicit none

    real*8, dimension(3), intent(in) :: s1, s2
    real*8, dimension(3) :: ds, dr

    ds(:) = s2(:) - s1(:)
    ds(1) = ds(1) - floor( ds(1) - 0.5d0 ) - 1
    ds(2) = ds(2) - floor( ds(2) - 0.5d0 ) - 1
    ds(3) = ds(3) - floor( ds(3) - 0.5d0 ) - 1
    dr = matmul( ds(:), box(:,:) )
    pbc_dr = dsqrt(dot_product( dr, dr ))
    return

  end function pbc_dr

end module routines
