module linklist
implicit none

contains

!********************************************************************************
! This subroutine construct a link list for neighbour searching
! For algorithm details, see Allen&Tildesley P149
! Major output: 
! integer, dimension(:,:,:,2) :: headlist  ! the head pointers for all cell
! integer, dimension(:,:,2) :: nslist      ! the link list for all the atoms
! Modified in 2012-11-08 to accomodate general box shape
!********************************************************************************
  subroutine construct_linklist(box,nx,ny,nz,tot_n_mole,n_mole,n_atom,xyz,headlist,nslist)
  use global_variables
  implicit none
    real*8, dimension(:,:), intent(in)::box 
    integer, intent(in) :: nx, ny, nz, tot_n_mole,n_mole
    integer, dimension(:), intent(in) :: n_atom
    real*8, dimension(:,:,:), intent(in) :: xyz
    integer, dimension(nx,ny,nz,2), intent(out) :: headlist
    integer, dimension(:,:,:), intent(out) :: nslist
    integer :: i_mole, i_atom, ix, iy, iz, i_end_mole, i_end_atom
    integer, dimension(nx,ny,nz,2) :: endlist
    real*8, dimension(3) :: rtmp
   
!   pointers all set to null
    headlist = 0
    nslist = 0
    endlist = 0
    
    do i_mole = n_mole+1, tot_n_mole
      do i_atom = 1, n_atom(i_mole)
        rtmp = matmul(xyz(i_mole,i_atom,:), inv_box(:,:))
        ix = floor(rtmp(1)*nx) + 1
        iy = floor(rtmp(2)*ny) + 1
        iz = floor(rtmp(3)*nz) + 1
!       shift if it is out of the box
        ix = ix - floor(dble(ix-1)/nx)*nx
        iy = iy - floor(dble(iy-1)/ny)*ny
        iz = iz - floor(dble(iz-1)/nz)*nz
        if ( headlist(ix,iy,iz,1) == 0 ) then
          headlist(ix,iy,iz,1) = i_mole
          headlist(ix,iy,iz,2) = i_atom
          endlist(ix,iy,iz,1) = i_mole
          endlist(ix,iy,iz,2) = i_atom
        else
          i_end_mole = endlist(ix,iy,iz,1)
          i_end_atom = endlist(ix,iy,iz,2)
          nslist(i_end_mole,i_end_atom,1) = i_mole
          nslist(i_end_mole,i_end_atom,2) = i_atom
          endlist(ix,iy,iz,1) = i_mole
          endlist(ix,iy,iz,2) = i_atom
        endif
      enddo
    enddo
 
  do ix = 1, nx
    do iy = 1, ny
      do iz = 1, nz
        if ( headlist(ix,iy,iz,1) /= 0 .and. headlist(ix,iy,iz,2) == 0 ) then
          print *, headlist(ix,iy,iz,1), headlist(ix,iy,iz,2) 
        endif
      enddo
    enddo
  enddo

  return
  end subroutine construct_linklist

end module linklist
