module simulation_parameters
implicit none

contains
  !*******************************
  !  this subroutine reads general simulation run parameters out of 
  !  the appropriate input file
  !  nothing fancy here, we separate parameters which are read as strings into
  !  "Simulation Methodology" group, and parameters which are read as
  !  either integers or real numbers into "Simulation Parameters" group
  !*******************************
  subroutine read_simulation_parameters( ifile_simpmt )
    use global_variables
    character(*), intent(in) :: ifile_simpmt
    integer :: file_h = 10 , inputstatus,ind,ind1
    character(10) :: param_string
    character(30) :: param_name,line
    real*8 :: param_number
    integer:: flag_energy_decomposition=0, flag_solute_cross_parameter_set=0, flag_C8_10_dispersion_terms=0, flag_electrostatic_type=0, flag_temperature=0, flag_too_close=0
    integer:: flag_lj_cutoff=0, flag_ewald_cutoff=0, flag_screen_type=0, flag_springconstant=0, flag_thole=0, flag_drude_simulation=0, flag_n_threads=0, flag_lj_bkghm=0,flag_lj_comb_rule=0,flag_C12_dispersion=0, flag_alpha_sqrt=0, flag_lj_asqrt=0, flag_cav_grid_a=0, flag_cav_grid_b=0, flag_cav_grid_c=0, flag_na_nslist=0, flag_nb_nslist=0, flag_nc_nslist=0, flag_orientation_try=0, flag_REL_THRSH=0, flag_ABS_THRSH=0, flag_BZ_CUTOFF=0, flag_pme_grid=0, flag_quantum_correct=0, flag_start_grid_a=0,flag_finish_grid_a=0,flag_start_grid_b=0,flag_finish_grid_b=0,flag_start_grid_c=0,flag_finish_grid_c=0, flag_Qeq_charges=0


    open(unit=file_h,file=ifile_simpmt,status="old")

    ! read Simulation_Methodology input parameters
    do
       Read(file_h,'(A)',Iostat=inputstatus) line
       If(inputstatus < 0) Exit
       ind=INDEX(line,'Simulation Methodology')
       IF(ind .NE. 0) Exit
    enddo

    do 
       Read(file_h,* ,Iostat=inputstatus) param_name, param_string 
       If(inputstatus < 0) Exit
       ind=INDEX(param_name,'Simulation')
       ind1=INDEX(param_string,'Param')
       IF( (ind .NE. 0) .and. (ind1 .NE. 0) ) Exit

       Select Case(param_name)
       Case("energy_decomposition")
          energy_decomposition = param_string
          flag_energy_decomposition=1
       Case("solute_cross_parameter_set")
          solute_cross_parameter_set = param_string
          flag_solute_cross_parameter_set=1
       Case("C8_10_dispersion_terms")
          C8_10_dispersion_terms = param_string
          flag_C8_10_dispersion_terms=1
       Case("C12_dispersion")
          C12_dispersion = param_string
          flag_C12_dispersion=1
       Case("electrostatic_type")
          electrostatic_type = param_string
          flag_electrostatic_type=1
       Case("lj_comb_rule")
          lj_comb_rule = param_string
          flag_lj_comb_rule=1
       Case("Qeq_charges")
          Qeq_charges = param_string
          flag_Qeq_charges = 1
       Case("Feynmann_Hibbs_correction")
          if ( param_string == 'yes' ) then
             quantum_correct = 1
          else
             quantum_correct = 0
          endif
          flag_quantum_correct = 1
       End Select

    enddo

    ! now read Simulation_Parameters, parameters

    do 
       Read(file_h,* ,Iostat=inputstatus) param_name, param_number 
       If(inputstatus < 0) Exit

       Select Case (param_name)
       Case("temperature")
          temp = param_number
          flag_temperature=1
       Case("too_close")
          too_close = param_number
          flag_too_close = 1
       Case("lj_bkghm")
          lj_bkghm = param_number
          flag_lj_bkghm = 1
       Case("lj_cutoff")
          lj_cutoff = param_number
          flag_lj_cutoff = 1
       Case("ewald_cutoff")
          ewald_cutoff = param_number
          flag_ewald_cutoff = 1
       Case("screen_type")
          screen_type = NINT( param_number )
          if ( (screen_type .NE. 0 ) .and. (screen_type .NE. 1 ) ) then
             stop "screen_type setting not recognized, should be 0 or 1"
          endif
          flag_screen_type = 1
       Case("springconstant")
          springcon = param_number * 1.889725989**3  ! this conversion factor is a.u.(e^2/Bohr^3) to e^2/A^3
          flag_springconstant = 1
       Case("thole")
          thole = param_number
          flag_thole = 1
       Case("drude_simulation")
          drude_simulation = NINT( param_number )
          if ( (drude_simulation .NE. 0) .and. (drude_simulation .NE. 1) ) then
             stop "drude_simulation setting not recognized, should be 0 or 1"
          endif
          flag_drude_simulation = 1 
       Case("n_threads")
          n_threads = NINT( param_number )
          flag_n_threads = 1
       Case("pme_grid")
          pme_grid = nint(param_number)
          flag_pme_grid = 1
       Case("alpha_sqrt")
          alpha_sqrt = param_number
          flag_alpha_sqrt = 1
       Case("lj_asqrt")
          lj_asqrt = param_number
          flag_lj_asqrt = 1
       Case("cav_grid_a")
          cav_grid_a = nint(param_number)
          flag_cav_grid_a = 1
       Case("cav_grid_b") 
          cav_grid_b = nint(param_number)
          flag_cav_grid_b = 1
       Case("cav_grid_c") 
          cav_grid_c = nint(param_number)
          flag_cav_grid_c = 1
       Case("start_grid_a") 
          start_grid_a = nint(param_number)
          flag_start_grid_a = 1
       Case("finish_grid_a") 
          finish_grid_a = nint(param_number)
          flag_finish_grid_a = 1
       Case("start_grid_b") 
          start_grid_b = nint(param_number)
          flag_start_grid_b = 1
       Case("finish_grid_b") 
          finish_grid_b = nint(param_number)
          flag_finish_grid_b = 1
       Case("start_grid_c") 
          start_grid_c = nint(param_number)
          flag_start_grid_c = 1
       Case("finish_grid_c") 
          finish_grid_c = nint(param_number)
          flag_finish_grid_c = 1
       Case("na_nslist")
          na_nslist = nint(param_number)
          flag_na_nslist = 1
       Case("nb_nslist")
          nb_nslist = nint(param_number)
          flag_nb_nslist = 1
       Case("nc_nslist")
          nc_nslist = nint(param_number)
          flag_nc_nslist = 1
       Case("orientation_try")
          orientation_try = nint(param_number)
          flag_orientation_try = 1
       Case("REL_THRSH")
          REL_THRSH = param_number
          flag_REL_THRSH = 1
       Case("ABS_THRSH")
          ABS_THRSH = param_number
          flag_ABS_THRSH = 1
       Case("BZ_CUTOFF")
          BZ_CUTOFF = param_number
          flag_BZ_CUTOFF = 1
       Case("quantum_correct")
          quantum_correct = nint(param_number)
          flag_quantum_correct = 1
       End Select

    enddo

    Close(file_h)

    ! now make sure all required variables were given values
    ! Simulation_Methodology section


    !*************************************************************************
    !****************  energy decomposition force field requires many specific run parameters
    !****************
    !*************************************************************************
    if( flag_energy_decomposition .eq. 0 ) then
       stop "variable energy_decomposition has not been given a value under the Simulation Methodology section in simulation parameter input file"
    else
       ! alot of run input parameters are specific to using an energy decomposition force field
       ! make sure these parameters are present if this is the case
       ! if not using energy_decomposition, just turn these parameters off
       Select Case(energy_decomposition)
       Case("yes")
          if( flag_solute_cross_parameter_set .eq. 0 ) then
             stop "variable cross_parameter_set has not been given a value under the Simulation Methodology section in simulation parameter input file"
          endif
          if( flag_C8_10_dispersion_terms .eq. 0 ) then
             stop "variable C8_10_dispersion_terms has not been given a value under the Simulation Methodology section in simulation parameter input file"
          endif
          ! if C12_dispersion isn't present, assume we use it if we are using C8, C10 terms as well
          if( flag_C12_dispersion .eq. 0 ) then
             if ( C8_10_dispersion_terms .eq. "no" ) then
                C12_dispersion = "no"
             else
                C12_dispersion = "yes"
             endif
          else
             Select Case(C12_dispersion)
             Case("yes")
                if ( C8_10_dispersion_terms .ne. "yes" ) then
                   stop "must use C8_C10_dispersion_terms='yes' in order to use C12 terms"
                endif
                ! only use C12 dispersion without explicit cross term exponents, if reading in two solute sets.  NOTE for one solute set, explicit_cross_term_exponents must be set to yes
                Select Case( solute_cross_parameter_set )
                Case("yes")
                   if ( explicit_cross_term_exponents .ne. "no" ) then
                      stop "if using C12_dispersion, explicit_cross_term_exponents must be set to 'no' in glob_v.f90"
                   endif
                end select
             End Select
          endif
       Case("no")
          ! here we are not using energy decomposition, some parameters are not necessary
          if( flag_solute_cross_parameter_set .eq. 0 ) then
             solute_cross_parameter_set="no"
          else
             Select Case(solute_cross_parameter_set)
             Case("yes")
                write(*,*) "setting of variable solute_cross_parameter_set will be overidden and set to 'no' as this is not an energy_decomposition simulation"
             End Select
             solute_cross_parameter_set="no"
          endif
          if( flag_C8_10_dispersion_terms .eq. 0 ) then
             C8_10_dispersion_terms="no"
          else
             Select Case(C8_10_dispersion_terms)
             Case("yes")
                write(*,*) "setting of variable C8_10_dispersion_terms will be overidden and set to 'no' as this is not an energy_decomposition simulation"
             End Select
             C8_10_dispersion_terms="no"
          endif

       End Select
    endif
    !*************************************************************************


    if( flag_electrostatic_type .eq. 0 ) then
       stop "variable electrostatic_type has not been given a value under the Simulation Methodology section in simulation parameter input file"
    endif
    ! Simulation_Parameters section
    if( flag_temperature .eq. 0 ) then
       stop "variable temperature has not been given a value under the Simulation Parameters section in simulation parameter input file"
    endif
    if( flag_too_close .eq. 0 ) then
       stop "variable too_close has not been given a value under the Simulation Parameters section in simulation parameter input file"
    endif
    if( flag_lj_bkghm .eq. 0 ) then
       stop "variable lj_bkghm has not been given a value, set to either '1' for buckingham or '2' for lennard jones"
    endif

    if ( flag_pme_grid == 0 ) then
       pme_grid = 100
       write(*,*) "Variable pme_grid has not been given a value, set to default value 100"
    endif
    if ( flag_alpha_sqrt == 0 ) then
       alpha_sqrt = 0.6
       write(*,*) "Variable alpha_sqrt has not been given a value, set to default value 0.6"
    endif
    if ( flag_lj_asqrt == 0 ) then
       lj_asqrt = 0.6
       write(*,*) "Variable lj_asqrt has not been given a value, set to default value 0.6"
    endif
    if ( flag_cav_grid_a == 0 ) then
       cav_grid_a = 100
       write(*,*) "Variable cav_grid_a has not been given a value, set to default value 100"
    endif
    if ( flag_cav_grid_b == 0 ) then
       cav_grid_b = 100
       write(*,*) "Variable cav_grid_b has not been given a value, set to default value 100"
    endif
    if ( flag_cav_grid_c == 0 ) then
       cav_grid_c = 100
       write(*,*) "Variable cav_grid_c has not been given a value, set to default value 100"
    endif

    ! start_grid_a, finish_grid_a, start_grid_b, finish_grid_b, etc are optional
    ! if these variables are present, then we just compute a section of the free energy grid
    ! base this on whether start_grid_a is present
    if ( flag_start_grid_a == 0 ) then
       partial_grid_computation = 0 
    else
       partial_grid_computation = 1 
    endif
    ! if we are only computing a section of the free energy grid, make sure the rest of the start/finish grid settings are present
    if ( partial_grid_computation == 1 ) then
       if ( ( flag_finish_grid_a == 0 ) .or. ( flag_start_grid_b == 0 ) .or. ( flag_finish_grid_b == 0 ) .or. ( flag_start_grid_c == 0 ) .or. ( flag_finish_grid_c == 0 ) ) then
          write(*,*) ""
          write(*,*) "variable start_grid_a is present in the simulation_parameters.pmt file"
          write(*,*) "this signals the code to only compute part of the free energy grid, "
          write(*,*) "however, not all of the start/finish grid settings are present "
          write(*,*) "Please make sure start_grid_a, finish_grid_a, start_grid_b, finish_grid_b,"
          write(*,*) "start_grid_c, finish_grid_c are all present "
          stop
       endif
       ! now make sure the settings of start_grid_a, finish_grid_a, etc. are reasonable
       if ( ( start_grid_a < 1 ) .or. ( start_grid_a > cav_grid_a ) .or. ( finish_grid_a < 1 ) .or. ( finish_grid_a > cav_grid_a ) .or. ( finish_grid_a < start_grid_a ) ) then
          write(*,*) "for the current setting of cav_grid_a, variables start_grid_a and "
          write(*,*) "finish_grid_a must be between ", 1 , cav_grid_a
          stop
       elseif ( ( start_grid_b < 1 ) .or. ( start_grid_b > cav_grid_b ) .or. ( finish_grid_b < 1 ) .or. ( finish_grid_b > cav_grid_b ) .or. ( finish_grid_b < start_grid_b ) ) then
          write(*,*) "for the current setting of cav_grid_b, variables start_grid_b and "
          write(*,*) "finish_grid_b must be between ", 1 , cav_grid_b
          stop
       elseif ( ( start_grid_c < 1 ) .or. ( start_grid_c > cav_grid_c ) .or. ( finish_grid_c < 1 ) .or. ( finish_grid_c > cav_grid_c ) .or. ( finish_grid_c < start_grid_c ) ) then
          write(*,*) "for the current setting of cav_grid_c, variables start_grid_c and "
          write(*,*) "finish_grid_c must be between ", 1 , cav_grid_c
          stop
       endif
    endif

    if ( flag_na_nslist == 0 ) then
       na_nslist = 20
       write(*,*) "Variable na_nslist has not been given a value, set to default value 20"
    endif
    if ( flag_nb_nslist == 0 ) then
       nb_nslist =20
       write(*,*) "Variable nb_nslist has not been given a value, set to default value 20"
    endif
    if ( flag_nc_nslist == 0 ) then
       nc_nslist = 20
       write(*,*) "Variable nc_nslist has not been given a value, set to default value 20"
    endif
    if ( flag_orientation_try == 0 ) then
       orientation_try = 2000
       write(*,*) "Variable orientation_try has not been given a value, set to default value 2000"
    endif
    if ( flag_REL_THRSH == 0 ) then
       REL_THRSH = 0.05d0
       write(*,*) "Variable REL_THRSH has not been given a value, set to default value 0.05"
    endif
    if ( flag_ABS_THRSH == 0 ) then
       ABS_THRSH = 3.0
       write(*,*) "Variable ABS_THRSH has not been given a value, set to default value 3.0"
    endif
    if ( flag_BZ_CUTOFF == 0 ) then
       BZ_CUTOFF = 100d0
       write(*,*) "Variable BZ_CUTOFF has not been given a value, set to default value 100.0"
    endif
    if ( flag_quantum_correct == 0 ) then
       quantum_correct = 0
    endif
    if ( flag_Qeq_charges == 0 ) then
       Qeq_charges = "no"
    endif
    ! we only allow options for lj_comb_rule parameter if there is no energy decomposition
    if( flag_lj_comb_rule .eq. 0 ) then    
       Select Case(energy_decomposition)
       Case("no")
          Select Case(lj_bkghm)
          Case(1)
             ! if we are using a bkghm force field
             stop "variable lj_comb_rule has not been given a value.  For bkghm force field, Set to either 'standard' for all geometric combination rules, or 'ZIFFF' for ZIF FF exponent combination rule"  
          Case(2)
             ! if we are using a lj force field, set to Lorentz-Berthelot as default
             write(*,*) ""
             write(*,*) "Lorentz-Berthelot combination rule will be used for the Lennard-Jones cross terms"  
             write(*,*) ""
             lj_comb_rule = "standard"  
          End Select
       End Select
    else
       Select Case(energy_decomposition)
       Case("yes")
          ! here lj_comb_rule parameter must be ZIFFF
          if ( lj_comb_rule .ne. "ZIFFF" ) then
             stop "for energy decomposition force field, lj_comb_rule parameter must be set to 'ZIFFF' "
          endif
       End Select
    endif
    if( flag_lj_cutoff .eq. 0 ) then
       stop "variable lj_cutoff has not been given a value under the Simulation Parameters section in simulation parameter input file"
    endif
    if( flag_ewald_cutoff .eq. 0 ) then
       stop "variable ewald_cutoff has not been given a value under the Simulation Parameters section in simulation parameter input file"
    endif
    if( flag_screen_type .eq. 0 ) then
       stop "variable screen_type has not been given a value under the Simulation Parameters section in simulation parameter input file"
    endif
    if( flag_springconstant .eq. 0 ) then
       Select Case(drude_simulation)
       Case(1)
          stop "variable springconstant has not been given a value under the Simulation Parameters section in simulation parameter input file"
       case default
          ! springcon won't be used, just set it to default value
          springcon = 0.1 * 1.889725989**3
       End Select
    endif
    if( flag_thole .eq. 0 ) then
       Select Case(drude_simulation)
       Case(1)
       stop "variable thole has not been given a value under the Simulation Parameters section in simulation parameter input file"
       case default
          ! thole won't be used, just set it to default value
          thole = 2.0
       End Select
    endif
    if( flag_drude_simulation .eq. 0 ) then
       stop "variable drude_simulation has not been given a value under the Simulation Parameters section in simulation parameter input file"
    endif
    if( flag_n_threads .eq. 0 ) then
       ! set to 1 for default
       n_threads = 1
    else
       ! right now OMP stuff is commented, so make sure n_threads=1
       if ( n_threads .ne. 1 ) then 
          write(*,*) "currently, all Open MP parallelization is commented in source code"
          write(*,*) "so please set n_threads = 1 "
          stop
       endif
    endif



    !********  check for conflicting run parameters for energy decomposition *********************!
    Select Case(energy_decomposition)
    Case("yes")
       ! energy decomposition only makes sense for buckingham
       if ( lj_bkghm .ne. 1 ) then
          stop "energy decomposition only makes sense for buckingham potential"
       endif
       ! must use C8_10_dispersion terms if we're reading in extra solute parameter set
       Select Case(solute_cross_parameter_set)
       Case("yes")
          Select Case(C8_10_dispersion_terms)
          Case("no")
             stop "must use C8_10_dispersion terms if reading in an extra solute parameter set"
          End Select
       End Select
    Case("no")
       !***************** no energy decomposition  ****************************!
       if ( lj_bkghm .eq. 1 ) then
          ! warn if ZIFFF combination rule is specified without energy decomposition
          if ( lj_comb_rule .eq. "ZIFFF" ) then
             write(*,*) "Are you sure you want to be using ZIF FF combination without an energy decomposition force field?"
          endif
       endif
    End Select
    !**********************************************************************************************!


  end subroutine read_simulation_parameters

end module simulation_parameters
