!******************************************
! this program is meant to run a Monte Carlo simulation for a lattice model of a 
! unit cell, in which the orientationaly averaged boltzmann factors for each
! lattice point are read in as input.  The solute - solute interaction is
! read in from a input file as well
!*****************************************

program lattice
  use routines
  use global_variables
  use sampling_routines

  character(MAX_FN) :: ifile_lattice, ifile_pairint
  character(5) :: input
  integer :: i_step, start_step
  real*8  :: tot_energy
  integer :: sum_molecules, n_input, nblocks
  real*8 :: average_part,average_density, sum_density

  n_input = IARGC()
  if (n_input .ne. 4 ) then
     stop " must have 4 command line arguments"
  endif

  ! first input file is free energy grid
  call getarg( 1, ifile_lattice )
  ! second input file is coarse-grained solute-solute potential grid
  call getarg( 2, ifile_pairint )
  ! third input is chemicall potential
  call getarg( 3, input )
  read(input,*) chem_potential
  ! last is mass, the reason we need mass is we want the chemical potential to be consistent with our
  ! atomistic MC code
  call getarg( 4, input )
  read(input,*) mass_solute


  call get_lattice_input( ifile_lattice )
  call initialize_lattice_data
  call initialize_pair_grid( ifile_pairint )

  ! start from zero molecules for now
  tot_energy=0d0
  n_particles=0
  sum_molecules=0

  write(*,*) "performing ", n_equil, " equilibration steps..."
  ! equilibrate
  do i_step = 1, n_equil
     call mc_move(tot_energy, sum_molecules)
  enddo
  write(*,*) "...done with equilibration"

  sum_molecules=0

  nblocks=0
  sum_density=0d0
  ! production run
  do i_step = 1, n_steps

     call mc_move(tot_energy, sum_molecules)

     if ( mod( i_step, n_output ) == 0 ) then
        average_part = dble(sum_molecules) / dble(n_output)
        average_density = average_part / tot_volume
        nblocks=nblocks+1
        sum_density=sum_density+average_density
        start_step = i_step - n_output
        write(*,*) "average density in molecules/Ang^3 from ", start_step," to ", i_step, " steps"
        write(*,*)  average_density
        sum_molecules=0
     endif
  enddo

  average_density = sum_density / dble(nblocks)

  write(*,*) ""
  write(*,*) "Final Average density from ", nblocks, " blocks is:(molecules/Ang^3)"
  write(*,*) average_density





end program lattice
