module routines
contains

  subroutine get_lattice_input( ifile_lattice )
    use global_variables
    character(*),intent(in) :: ifile_lattice

    real*8 :: tot_v_test,diff_v
    real*8,parameter :: diff_v_thresh = 1.
    character(50)::line
    integer :: file_h=7,inputstatus,ind,i,j,k
    integer,dimension(3) :: test_index
    character(25):: junk


    open(unit=file_h,file=ifile_lattice,status='old')

    !*********** get temperature, volume, grid size, grid volume
    do
       Read(file_h,'(A)',Iostat=inputstatus) line
       If(inputstatus < 0) Exit
       ind=INDEX(line,'temperature')
       IF(ind .NE. 0) Exit
    enddo
    backspace(file_h)
    read(file_h,*) junk, temp
    do
       Read(file_h,'(A)',Iostat=inputstatus) line
       If(inputstatus < 0) Exit
       ind=INDEX(line,'volume')
       IF(ind .NE. 0) Exit
    enddo
    backspace(file_h)
    read(file_h,*) junk, tot_volume
    do
       Read(file_h,'(A)',Iostat=inputstatus) line
       If(inputstatus < 0) Exit
       ind=INDEX(line,'lattice grid')
       IF(ind .NE. 0) Exit
    enddo
    backspace(file_h)
    read(file_h,'(A22,3(9x,I3))') junk, grid_size(1),grid_size(2),grid_size(3)
    do
       Read(file_h,'(A)',Iostat=inputstatus) line
       If(inputstatus < 0) Exit
       ind=INDEX(line,'cubelet_volume')
       IF(ind .NE. 0) Exit
    enddo
    backspace(file_h)
    read(file_h,*) junk, cube_volume
  !********************** check to make sure tot_volume, cube_volume, and grid size are consistent

    tot_v_test = cube_volume * dble(grid_size(1)) * dble(grid_size(2)) * dble(grid_size(3))
    diff_v = abs(tot_v_test - tot_volume ) 

    if ( diff_v > diff_v_thresh ) then
       stop " tot_volume, cube_volume, and grid size are not consistent"
    endif

  ! ****************** allocate grid arrays
    allocate( xyz_grid ( grid_size(1) , grid_size(2) , grid_size(3), 3 ) )
    allocate( avg_boltz_grid ( grid_size(1) , grid_size(2) , grid_size(3) ) )
    allocate( lattice_energy ( grid_size(1) , grid_size(2) , grid_size(3) ) )

  ! ********************* get averaged boltzmann factor for all grid points

    do
       Read(file_h,'(A)',Iostat=inputstatus) line
       If(inputstatus < 0) Exit
       ind=INDEX(line,'cavity index')
       IF(ind .NE. 0) Exit
    enddo
    backspace(file_h)

    write(*,*) "reading cube data.........."

    do i=1, grid_size(1)
       do j=1, grid_size(2)
          do k=1, grid_size(3)
             read(file_h,*) junk,junk, test_index(1), test_index(2), test_index(3)
             ! check to make sure indices are correct
             if ( ( i .ne. test_index(1) ) .or. ( j .ne. test_index(2) ) .or. ( k .ne. test_index(3) ) ) then
                stop "input file format is inconsistent with subroutine get_lattice_input "
             endif

             read(file_h,*) xyz_grid(i,j,k,1) , xyz_grid(i,j,k,2) , xyz_grid(i,j,k,3)
             read(file_h,*) line
             read(file_h,*) line
             read(file_h,*) avg_boltz_grid(i,j,k)
             
             enddo
          enddo
       enddo

       close(file_h)

    write(*,*) " finished reading cube data"

end subroutine get_lattice_input


!***********************************
! this subroutine creates the lattice-solute interaction energies from
! the orientationally averaged boltzmann factors that were read in
! 
! since the derivative of the log of this number with respect to lattice index is the mean
! force required to move to an adjacent lattice ( where mean refers to orientations being averaged out)
! we define the interaction energy as Beta times the log of this averaged boltzmann factor
!***********************************
subroutine initialize_lattice_data
  use global_variables

  real*8 :: inv_beta
  integer  :: i,j,k

! first do energy

  inv_beta = (8.314 * temp)/1000.

do i=1, grid_size(1)
   do j=1, grid_size(2)
      do k=1, grid_size(3)
         lattice_energy(i,j,k) = -inv_beta * log( avg_boltz_grid(i,j,k))
      enddo
   enddo
enddo



end subroutine initialize_lattice_data




!**************************************************************************************
! This subroutine read the pairwise interaction grid from file ifn
!**************************************************************************************
  subroutine initialize_pair_grid( ifn )
  use global_variables
  implicit none

    character(100), intent(in) :: ifn
    integer, parameter :: ifile=99
    integer :: ia, ib, ic, i, j, k
    character(100) :: line

    allocate( pair_int_grid( grid_size(1), grid_size(2), grid_size(3) ) )
    pair_int_grid = 0.0d0
    open( ifile, file=ifn, status='old' )

    do ia = 1, grid_size(1)
      do ib = 1, grid_size(2)
        do ic = 1, grid_size(3)
          read(ifile,*) i, j, k, pair_int_grid( i, j, k )
          if ( i/=ia .or. j/=ib .or. k/=ic ) then
            write(*,*) 'Error in reading pairwise interaction grid:', ia, ib, ic
          endif
        enddo
        read(ifile,'(A)') line
      enddo
    enddo

    close(ifile)
    return

  end subroutine initialize_pair_grid

  function pair_int( i_lattice, j_lattice )
  use global_variables 
  implicit none
    integer, intent(in), dimension(3) :: i_lattice, j_lattice
    integer, dimension(3) :: di_lattice
    real*8 :: pair_int


! reading from file
    di_lattice = j_lattice - i_lattice + 1
    if ( di_lattice(1) <= 0 ) di_lattice(1) = di_lattice(1) + grid_size(1)
    if ( di_lattice(2) <= 0 ) di_lattice(2) = di_lattice(2) + grid_size(2)
    if ( di_lattice(3) <= 0 ) di_lattice(3) = di_lattice(3) + grid_size(3)
    pair_int = pair_int_grid( di_lattice(1), di_lattice(2), di_lattice(3) )

    return
  end function pair_int 

!*************************************************************************
! This subroutine update the energy of the whole lattice
! if i_part >= 1 and i_lattice != 0 then it is a particle moving update
! if i_part >= 1 and i_lattice == 0 then it is particle deletion update
! if i_part == 0 and i_lattice != 0 then it is a particle insertion update
!*************************************************************************
  subroutine potent_update( energy_try, energy, lattice_index, i_lattice, i_part)
  use global_variables
  implicit none
    real*8, intent(out) :: energy_try
    real*8, intent(in) :: energy
    integer, intent(in), dimension(MAX_N_PART,3) :: lattice_index
    integer, intent(in), dimension(3) :: i_lattice
    integer, intent(in) :: i_part
    integer :: i_sol

    energy_try = energy
!   --- deletion
    if ( i_part >= 1 ) then
      do i_sol = 1, n_particles
        if ( i_sol /= i_part ) then
          energy_try = energy_try - pair_int( lattice_index(i_part,:), lattice_index(i_sol,:) )
        endif
      enddo
      energy_try = energy_try - lattice_energy( lattice_index(i_part,1), lattice_index(i_part,2), lattice_index(i_part,3) )
    endif
!   --- insertion
    if ( i_lattice(1) > 0 ) then
      do i_sol = 1, n_particles
        if ( i_sol /= i_part ) then
          energy_try = energy_try + pair_int( i_lattice(:), lattice_index(i_sol,:) )
        endif
      enddo
      energy_try = energy_try + lattice_energy( i_lattice(1), i_lattice(2), i_lattice(3) )
    endif

  end subroutine potent_update

end module routines
