module sampling_routines
use routines

contains 

!***********************************
! this subroutine determines whether to make a lattice move, or try insertion/deletion
!**********************************
subroutine mc_move(tot_energy,sum_molecules)
  use global_variables
  real*8,intent(inout) :: tot_energy
  integer,intent(inout) :: sum_molecules
  real*8 :: randnum



  call random_number(randnum)

  if( randnum .ge. gcmc_ins_rm ) then
     call lattice_move(tot_energy)
  else
     call lattice_ins_del(tot_energy)
  endif

  ! keep running average of number of particles

  sum_molecules = sum_molecules + n_particles


end subroutine mc_move


!**********************************
! this subroutine makes lattice moves
! currently, it randomly selects a position from the entire
! lattice for each move
!**********************************
subroutine lattice_move(energy)
  use global_variables
  real*8,intent(inout) :: energy

  real*8 :: randnum, energy_try,accept_crit,arg
  integer :: i_part,i
  integer,dimension(3) :: i_lattice
  
! if zero particle, exit this subroutine
  if ( n_particles .eq. 0 ) then
     goto 100
  endif


! pick a molecule
  call random_number(randnum)
  i_part=int(randnum * n_particles) + 1

! pick a new lattice site at random
  do i=1,3
    call random_number(randnum)
    i_lattice(i) = int(randnum * grid_size(i)) + 1
  enddo


  call potent_update( energy_try, energy, particle_lattice_index, i_lattice, i_part )

  arg = -( energy_try - energy ) * 1000. / ( 8.314 * temp )
  accept_crit = min(1.,exp(arg))

  call random_number(randnum)

  if ( randnum < accept_crit ) then
    energy = energy_try
    particle_lattice_index(i_part,:) = i_lattice(:)
  endif

100 continue

end subroutine lattice_move



!**********************************
! this subroutine makes lattice insertions and deletions
!**********************************
subroutine lattice_ins_del(energy)
  use global_variables
  real*8,intent(inout) :: energy

  real*8 :: randnum, energy_try,accept_crit,arg,lambda,fac
  integer :: r_mole,i,insert
  integer,dimension(3) :: i_lattice
  integer,dimension(MAX_N_PART,3) :: temp_particle_lattice_index
  integer  :: n_particles_store

  n_particles_store = n_particles

  !!decide whether to insert or remove a molecule, insert=0 means remove, =1 means insert

  call random_number(randnum)
  if(randnum .lt. .5 ) then
     insert=0
     if( n_particles .eq. 0) then
        ! important to reject this deletion at zero molecules, rather than just automatically insert at zero molecules, otherwise violate detailed balance
        goto 200
     endif
  else
     insert=1
  endif

  i_lattice = 0
  r_mole = 0
  ! a molecule is being removed
  if ( insert .eq. 0) then
     call random_number(randnum)
     r_mole=ceiling(randnum*real(n_particles))
     !     n_particles = n_particles-1
     ! copy array
     do i=1,n_particles
        temp_particle_lattice_index(i,:) = particle_lattice_index(i,:)
     enddo

     do i = r_mole , n_particles-1
        temp_particle_lattice_index(i,:) = temp_particle_lattice_index(i+1,:)
     enddo

  else
     ! molecule is inserted
     !     n_particles = n_particles+1

     ! pick a new lattice site at random
     do i=1,3
        call random_number(randnum)
        i_lattice(i) = int(randnum * grid_size(i)) + 1
     enddo

     ! copy array
     do i=1,n_particles
        temp_particle_lattice_index(i,:) = particle_lattice_index(i,:)
     enddo

     temp_particle_lattice_index(n_particles+1,:) = i_lattice(:)

  endif

  ! --- Modified by Kuang
  ! Note that this is correct inputing particle_lattice_index array here, instead of temp_particle_lattice_index
  ! this is because for a deletion, subroutine needs the lattice indices of r_mole which are found in particle_lattice_index array
  ! while for an insertion, this array isn't used, rather the lattice indices of the new particle are taken from i_lattice array
  call potent_update( energy_try, energy, particle_lattice_index, i_lattice, r_mole )
  if ( insert .eq. 0) then
     n_particles = n_particles-1
  else
     n_particles = n_particles+1
  endif

  lambda= 126.178D0/sqrt(2D0*pi*mass_solute*8.314D0*temp)

  if (insert .eq. 0) then
     fac= lambda**3 * dble(n_particles_store) / tot_volume* exp(-1000./(temp*8.314)*chem_potential)
  else
     fac= tot_volume /(lambda**3 * dble(n_particles))* exp(1000./(temp*8.314)*chem_potential) 
  endif

  arg=-(1000./(temp*8.314))*(energy_try-energy)
  accept_crit=min(1.,fac*exp(arg))

  call random_number(randnum)

  if ( randnum < accept_crit ) then
     energy = energy_try
     ! copy array
     do i=1,n_particles
        particle_lattice_index(i,:) = temp_particle_lattice_index(i,:)
     enddo
  else
     n_particles = n_particles_store
  endif


200 continue

end subroutine lattice_ins_del

end module sampling_routines
